// The xMule Project - A Peer-2-Peer File Sharing Program
//
// Copyright (C) 2003-2006 Theodore R. Smith ( hopeseekr@gmail.com / http://www.xmule.ws/ )
// Copyright (C) 2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of Version 2 of the GNU General Public
// License as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

#ifdef PRECOMP
    #include "xmule-headers.h"
#endif

#include "ServerWnd.h"
#include "HTTPDownloadDlg.h"
#include "muuli_wdr.h"
#include "wintypes.h"
#include "xmule.h"
#include "xmuleDlg.h"
#include "otherfunctions.h"       // GetResString
#include "ServerList.h"           // CServerList
#include "ServerListCtrl.h"       // CServerListCtrl

#include <DynPrefs/DynPrefs.h>    // DynamicPreferences
#include <wx/settings.h>
#include <wx/sizer.h>             // wxSizer
#include <wx/textctrl.h>          // wxTextCtrl
#include <sstream>
#include <iostream>

#ifdef HAVE_CONFIG_H
    #include <config.h>
#endif // HAVE_CONFIG_H

using std::cout;
using std::endl;

BEGIN_EVENT_TABLE(CServerWnd, wxPanel)
    EVT_BUTTON(ID_ADDTOLIST, CServerWnd::OnBnClickedAddserver)
    EVT_BUTTON(ID_UPDATELIST, CServerWnd::OnBnClickedUpdateservermetfromurl)
    EVT_BUTTON(ID_BTN_RESET, CServerWnd::OnBnClickedResetLog)
END_EVENT_TABLE()

CServerWnd::CServerWnd(wxWindow *pParent) : wxPanel(pParent, CServerWnd::IDD)
{
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWFRAME));
    wxSizer *content = serverListDlg(this, true);
    content->Show(this, true);
    // init serverlist
    // no use now. too early.
    CServerListCtrl *list = (CServerListCtrl *) wxFindWindowByName(wxT("serverListCtrl"));
    //list->Init(theApp.serverlist);
    serverlistctrl = list;
    wxTextCtrl *cv = (wxTextCtrl *) FindWindowByName(wxT("serverInfo"));
    std::ostringstream internalbuffer;
    internalbuffer << "This is " << "xMule" << " " << VERSION << " (based on " << "eMule" << ")" << endl;
    cv->AppendText(wxString(internalbuffer.str().c_str(), *wxConvCurrent));
    cv->AppendText(wxString(_("Visit http://www.xmule.ws/ to check if a new version is available.\n")));
    //  cv->AppendText(GetResString(IDS_EMULEW)+wxString(" "));
    //  cv->AppendText(GetResString(IDS_EMULEW3)+wxString(" "));
    //  cv->AppendText(GetResString(IDS_EMULEW2)+wxString("\n\n"));
}

CServerWnd::~CServerWnd()
{
}

void CServerWnd::UpdateServerMetFromURL(wxString strURL)
{
    // not a valid URL:
    if (strURL.Find(wxT("://")) == - 1)
    {
        theApp.xmuledlg->AddLogLine(true, GetResString(IDS_INVALIDURL));
        return;
    }
    wxString strTempFilename;
    strTempFilename.Printf(DynPrefs::Get<wxString>("app-directory") + wxT("temp-%d-server.met"), ::GetTickCount());
    CHTTPDownloadDlg *dlg = new CHTTPDownloadDlg(this, strURL, strTempFilename);
    int retval = dlg->ShowModal();
    if (retval == 0)
    {
        // wget succeeded. proceed with serverlist processing
        serverlistctrl->AddServermetToList(strTempFilename);
        wxRemoveFile(strTempFilename);
        theApp.serverlist->SaveServermetToFile();
    }
    else
    {
        theApp.xmuledlg->AddLogLine(true, GetResString(IDS_ERR_FAILEDDOWNLOADMET) .GetData(), strURL.GetData());
    }
    delete dlg;
}

void CServerWnd::Localize()
{
}

// CServerWnd message handlers

void CServerWnd::OnBnClickedAddserver(wxCommandEvent &evt)
{
    wxString serveraddr;
    if (((wxTextCtrl *) FindWindowByName(wxT("ipAddress")))->GetLineText(0) .IsEmpty())
    {
        //if (!GetDlgItem(IDC_IPADDRESS)->GetWindowTextLength()){
        theApp.xmuledlg->AddLogLine(true, GetResString(IDS_SRV_ADDR));
        return;
    }
    else
    serveraddr = ((wxTextCtrl *) FindWindowByName(wxT("ipAddress")))->GetLineText(0);
    //GetDlgItem(IDC_IPADDRESS)->GetWindowText(serveraddr);
    //if (!GetDlgItem(IDC_SPORT)->GetWindowTextLength()){
    if (((wxTextCtrl *) FindWindowByName(wxT("portNumber")))->GetLineText(0) .IsEmpty())
    {
        theApp.xmuledlg->AddLogLine(true, GetResString(IDS_SRV_PORT));
        return;
    }
    //char portstr[6];
    //GetDlgItem(IDC_SPORT)->GetWindowText(portstr,6);
    wxString portstr;
    portstr = ((wxTextCtrl *) FindWindowByName(wxT("portNumber")))->GetLineText(0);
    CServer *toadd = new CServer(atoi(portstr.mb_str(*wxConvCurrent)), serveraddr.mb_str(*wxConvCurrent));
    //int32_t namelen = GetDlgItem(IDC_SNAME)->GetWindowTextLength();
    wxString servername;
    servername = ((wxTextCtrl *) FindWindowByName(wxT("serverName")))->GetLineText(0);
    if (!servername.IsEmpty())
    {
        //char* servername = new char[namelen+2];
        //GetDlgItem(IDC_SNAME)->GetWindowText(servername,namelen+1);
        toadd->SetListName(servername.mb_str(*wxConvCurrent));
        //delete[] servername;
    }
    else
    {
        toadd->SetListName(serveraddr.mb_str(*wxConvCurrent));
    }
    if (!serverlistctrl->AddServer(toadd, true))
    {
        CServer *update = theApp.serverlist->GetServerByAddress(toadd->GetAddress(), toadd->GetPort());
        if (update)
        {
            update->SetListName(toadd->GetListName());
            serverlistctrl->RefreshServer(update);
        }
        delete toadd;
        theApp.xmuledlg->AddLogLine(true, GetResString(IDS_SRV_NOTADDED));
    }
    else theApp.xmuledlg->AddLogLine(true, GetResString(IDS_SERVERADDED) + wxT("%s"), wxString(toadd->GetListName(), *wxConvCurrent).GetData());
    theApp.serverlist->SaveServermetToFile();
}

void CServerWnd::OnBnClickedUpdateservermetfromurl(wxCommandEvent &evt)
{
    wxString strURL;
    strURL = ((wxTextCtrl *) FindWindowByName(wxT("serverlistURL")))->GetLineText(0);
    UpdateServerMetFromURL(strURL);
}

void CServerWnd::OnBnClickedResetLog(wxCommandEvent &evt)
{
    theApp.xmuledlg->ResetLog();
}

void CServerWnd::UpdateMyInfo()
{
    //	TODO: MyInfo @ CServerWnd (GUI missing)
}
